/*
 * BridJ - Dynamic and blazing-fast native interop for Java.
 * http://bridj.googlecode.com/
 *
 * Copyright (c) 2010-2013, Olivier Chafik (http://ochafik.com/)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of Olivier Chafik nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY OLIVIER CHAFIK AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.bridj;

import org.bridj.ann.Constructor;

/**
 * Base class for C structs.
 *
 * @author Olivier
 */
@org.bridj.ann.Runtime(CRuntime.class)
public abstract class StructObject extends NativeObject {

    protected StructIO io;

    protected StructObject() {
        super();
    }

    /**
     * Identified constructor with an arbirary number of arguments
     *
     * @param voidArg always null, here to disambiguate some sub-constructors
     * @param constructorId identifier of the constructor, has to match a
     * {@link Constructor} annotation or be -1.
     * @param args
     */
    protected StructObject(Void voidArg, int constructorId, Object... args) {
        super(constructorId, args);
    }

    protected StructObject(Pointer<? extends StructObject> peer, Object... targs) {
        super(peer, targs);
    }

    /**
     * Creates a String out of this struct using BridJ.describe(this) (see {@link BridJ#describe(NativeObject)
     * }).
     */
    @Override
    public String toString() {
        return BridJ.describe(this);
    }
    
    /**
     * Get the offset of a field in a struct.
     */
    public static long offsetOfField(StructObject o, String name) {
        for (StructFieldDescription desc : o.io.desc.fields) {
            if (desc.name.equals(name)) {
                return desc.byteOffset;
            }
        }
        throw new NoSuchFieldError(name);
    }
}
